import { useRef, useState } from 'react'
import { MarkInputProps } from '../../types/mark'
import { getOptimizedPaths, optimizeSVG } from '../../utils'
import { faEraser } from '@fortawesome/free-solid-svg-icons'
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome'
import styles from './Signature.module.scss'
import { MarkRenderSignature } from '../MarkRender/Signature'

export const MarkInputSignature = ({
  value,
  handler,
  userMark
}: MarkInputProps) => {
  const location = userMark?.mark.location

  const canvasRef = useRef<HTMLCanvasElement>(null)
  const [drawing, setDrawing] = useState(false)
  const [paths, setPaths] = useState<string[]>(value ? JSON.parse(value) : [])

  function update() {
    if (location && paths) {
      if (paths.length) {
        const optimizedSvg = optimizeSVG(location, paths)
        const extractedPaths = getOptimizedPaths(optimizedSvg)
        handler(JSON.stringify(extractedPaths))
      } else {
        handler('')
      }
    }
  }

  const handlePointerDown = (event: React.PointerEvent) => {
    const rect = event.currentTarget.getBoundingClientRect()
    const x = event.clientX - rect.left
    const y = event.clientY - rect.top

    const ctx = canvasRef.current?.getContext('2d')
    ctx?.beginPath()
    ctx?.moveTo(x, y)
    setPaths([...paths, `M ${x} ${y}`])
    setDrawing(true)
  }
  const handlePointerUp = () => {
    setDrawing(false)
    update()
    const ctx = canvasRef.current?.getContext('2d')
    ctx?.clearRect(0, 0, canvasRef.current!.width, canvasRef.current!.height)
  }
  const handlePointerMove = (event: React.PointerEvent) => {
    if (!drawing) return
    const ctx = canvasRef.current?.getContext('2d')
    const rect = canvasRef.current?.getBoundingClientRect()
    const x = event.clientX - rect!.left
    const y = event.clientY - rect!.top

    ctx?.lineTo(x, y)
    ctx?.stroke()

    // Collect the path data
    setPaths((prevPaths) => {
      const newPaths = [...prevPaths]
      newPaths[newPaths.length - 1] += ` L ${x} ${y}`
      return newPaths
    })
  }

  const handleReset = () => {
    setPaths([])
    setDrawing(false)
    update()
    const ctx = canvasRef.current?.getContext('2d')
    ctx?.clearRect(0, 0, canvasRef.current!.width, canvasRef.current!.height)
  }

  return (
    <>
      <div className={styles.wrapper}>
        <div className={styles.relative}>
          <canvas
            height={location?.height}
            width={location?.width}
            ref={canvasRef}
            className={styles.canvas}
            onPointerDown={handlePointerDown}
            onPointerUp={handlePointerUp}
            onPointerMove={handlePointerMove}
            onPointerOut={handlePointerUp}
          ></canvas>
          {typeof userMark?.mark !== 'undefined' && (
            <div className={styles.absolute}>
              <MarkRenderSignature value={value} mark={userMark.mark} />
            </div>
          )}
          <div className={styles.reset}>
            <FontAwesomeIcon size="sm" icon={faEraser} onClick={handleReset} />
          </div>
        </div>
      </div>
    </>
  )
}
